@@ -5,6 +5,24 @@ import warnings
 #
 #*****************************************************************************
 #
+class _CommonTools():
+    def __init__(self, awrde):#----------------------------------------------------------
+        self.awrde = awrde
+        #
+    @property
+    def get_all_active_measurements(self) -> list:#------------------------------------
+        '''returns a list of all the active measurements from all the graphs in the project'''
+        active_meas_list = list()
+        for gr in self.awrde.Project.Graphs:
+            for meas in gr.Measurements:
+                active_meas_list.append(meas.Name)
+            #end for
+        #end for
+        return active_meas_list
+        #    
+#
+#*****************************************************************************
+#
 class _Document_Methods():
     def __init__(self, document: object,):#--------------------------------------------
         self._document = document
@@ -98,18 +116,18 @@ class _Parameter():
         self._document_type = document_type
 
     @property
-    def name(self):
+    def name(self) -> str:
         return self._parameter.Name
 
     @property
-    def value(self):
+    def value(self):#-------------------------------------------------------------
         if self._document_type == 'System':
             raise RuntimeError('Only value_str is valid for System Diagram element parameters')
         #end if
         return self._parameter.ValueAsDouble
 
     @value.setter
-    def value(self, value):
+    def value(self, value):#-----------------------------------------------------
         if not (isinstance(value, float) or isinstance(value, int)):
             raise RuntimeError('value must be float or int type')
         #end if        
@@ -117,21 +135,91 @@ class _Parameter():
             raise RuntimeError('Only value_str is valid for System Diagram element parameters')
         #end if
         self._parameter.ValueAsDouble = value
-
+        #
     @property
-    def value_str(self):
+    def value_str(self) -> str:#-----------------------------------------------------
         return self._parameter.ValueAsString
-
+        #
     @value_str.setter
-    def value_str(self, value_str: str):
+    def value_str(self, value_str: str):#-------------------------------------------
         if not isinstance(value_str, str):
             raise RuntimeError('value_str must be string type')
         #end if
         self._parameter.ValueAsString = value_str
-
-
+        #
+    @property
+    def value_float(self) -> float:#--------------------------------------------------
+        '''For system diagrams where value string cannot be used'''
+        return self._parameter.ValueAsDouble
+        #
+    @value_float.setter
+    def value_float(self, value_float: float):#------------------------------------------
+        if not isinstance(value_float, float):
+            raise TypeError('value_float must be float type')
+        #end if
+        self._parameter.ValueAsDouble = value_float
+        #
+    @property
+    def optimize_enabled(self) -> bool:#----------------------------------------------
+        '''returns Optimize enabled state '''
+        return self._parameter.Optimize
+        #
+    @optimize_enabled.setter
+    def optimize_enabled(self, enabled: bool):#--------------------------------------
+        '''sets Optimize enabled state '''
+        self._parameter.Optimize = enabled
+        #
+    @property
+    def constrain(self) -> bool:#-------------------------------------------------------
+        '''returns Contrain enabled state '''
+        return self._parameter.Constrain
+        #
+    @constrain.setter
+    def constrain(self, enabled: bool):#----------------------------------------------------
+        '''sets Contrain enabled state '''
+        self._parameter.Constrain = enabled
+        #
+    @property
+    def lower_constraint(self) -> float:#-------------------------------------------------
+        ''' returns LowerContraint value'''
+        return self._parameter.LowerConstraint
+        #
+    @lower_constraint.setter
+    def lower_constraint(self, lower_contraint_value:float):#-----------------------------
+        ''' sets LowerContraint value'''
+        if not isinstance(lower_contraint_value, float) and  not isinstance(lower_contraint_value, int):
+            raise TypeError('lower_contraint_value must be float or int')
+        #end if
+        self._parameter.LowerConstraint = lower_contraint_value
+        #
+    @property
+    def upper_constraint(self) -> float:#-------------------------------------------------
+        ''' returns UpperContraint value'''
+        return self._parameter.UpperConstraint
+        #
+    @upper_constraint.setter
+    def upper_constraint(self, upper_contraint_value:float):#-----------------------------
+        ''' sets UpperContraint value'''
+        if not isinstance(upper_contraint_value, float) and  not isinstance(upper_contraint_value, int):
+            raise TypeError('upper_contraint_value must be float or int')
+        #end if
+        self._parameter.UpperConstraint = upper_contraint_value
+        #
+    @property
+    def step_size(self) -> float:#-------------------------------------------------
+        ''' returns StepSize value'''
+        return self._parameter.StepSize
+        #
+    @step_size.setter
+    def step_size(self, step_size_value:float):#-----------------------------
+        ''' sets StepSize value'''
+        if not isinstance(step_size_value, float) and  not isinstance(step_size_value, int):
+            raise TypeError('step_size_value must be float or int')
+        #end if
+        self._parameter.StepSize = step_size_value
+        #    
 class _Parameters():
-    def __init__(self, element, document_type):
+    def __init__(self, element, document_type):#----------------------------------------------
         self._element = element
         self._document_type = document_type
 
@@ -184,12 +272,24 @@ class _Elements():
 
     def _build_elements_dict(self):
         self._elements_dict = {}
-        for element in self._document.Elements:
-            try:
-                self._elements_dict[element.Name] = _Element(self._document.Elements(element.Name), self._document_type)
-            except:
-                raise RuntimeError('Failed to create dictionary of Elements')
-
+        if self._document_type == 'Global Definition':
+            for element in self._document.DataElements:
+                try:
+                    self._elements_dict[element.Name] = _Element(self._document.DataElements(element.Name), self._document_type)
+                except:
+                    raise RuntimeError('Failed to create dictionary of Elements')
+                #end try
+            #end for
+        else:
+            for element in self._document.Elements:
+                try:
+                    self._elements_dict[element.Name] = _Element(self._document.Elements(element.Name), self._document_type)
+                except:
+                    raise RuntimeError('Failed to create dictionary of Elements')
+                #end try
+            #end for
+        #end if
+        #
     @property
     def elements_dict(self) -> dict:
         return self._elements_dict
@@ -209,9 +309,15 @@ class _Elements():
                    Each item in the list is an element name
         '''
         elements_list = list()
-        for element in self._document.Elements:
-            elements_list.append(element.Name)
-
+        if self._document_type == 'Global Definition':
+            for element in self._document.DataElements:
+                elements_list.append(element.Name)
+            #end for
+        else:
+            for element in self._document.Elements:
+                elements_list.append(element.Name)
+            #end for
+        #end if
         return elements_list
 
     def add_element(self, element_name, x_pos: float, y_pos: float, rotation: float=0, flipped: bool=False):
@@ -317,12 +423,17 @@ class _Element(_Parameters):
         self._element.x = xy_position_tuple[0]
         self._element.y = xy_position_tuple[1]
         #
-    def element_enable(self):#------------------------------------------------------
-        self._element.Enabled = True
+    @property
+    def element_enabled(self) -> bool:#------------------------------------------------------       
+        return self._element.Enabled
+        #
+    @element_enabled.setter
+    def element_enabled(self,  enabled_state: bool):#-----------------------------------------
+        if not isinstance(enabled_state, bool):
+            raise TypeError('enabled_state must be boolean type')
+        #end if        
+        self._element.Enabled = enabled_state
         #
-    def element_disable(self):#------------------------------------------------------
-        self._element.Enabled = False
-        #    
     @property
     def element_nodes_dict(self) -> dict:#--------------------------------------------------------
         ''' 
@@ -533,6 +644,17 @@ class _Equation():
         '''returns equation name'''
         return self._equation_name
         #
+    @property
+    def equation_enabled(self) -> bool:#------------------------------------------
+        return self._eqn_doc.Equations(self._eqn_index).Enabled
+        #
+    @equation_enabled.setter
+    def equation_enabled(self, enabled_state: bool):#--------------------------
+        if not isinstance(enabled_state, bool):
+            raise TypeError('enabled_state must be boolean type')
+        #end if
+        self._eqn_doc.Equations(self._eqn_index).Enabled = enabled_state
+        #
     @equation_name.setter
     def equation_name(self, equation_name: str):
         '''modify equation name'''
@@ -599,6 +721,65 @@ class _Equation():
         self._eqn_doc.Equations(self._eqn_index).x = xy_position_tuple[0]
         self._eqn_doc.Equations(self._eqn_index).y = xy_position_tuple[1]
         #
+    @property
+    def optimize_enabled(self) -> bool:#----------------------------------------------
+        '''returns Optimize enabled state '''
+        return self._eqn_doc.Equations(self._eqn_index).Optimize
+        #
+    @optimize_enabled.setter
+    def optimize_enabled(self, enabled: bool):#--------------------------------------
+        '''sets Optimize enabled state '''
+        self._eqn_doc.Equations(self._eqn_index).Optimize = enabled
+        #
+    @property
+    def constrain(self) -> bool:#-------------------------------------------------------
+        '''returns Contrain enabled state '''
+        return self._eqn_doc.Equations(self._eqn_index).Constrain
+        #
+    @constrain.setter
+    def constrain(self, enabled: bool):#----------------------------------------------------
+        '''sets Contrain enabled state '''
+        self._eqn_doc.Equations(self._eqn_index).Constrain = enabled
+        #    
+    @property
+    def lower_constraint(self) -> float:#-------------------------------------------------
+        ''' returns LowerContraint value'''
+        return self._eqn_doc.Equations(self._eqn_index).LowerConstraint
+        #
+    @lower_constraint.setter
+    def lower_constraint(self, lower_contraint_value:float):#-----------------------------
+        ''' sets LowerContraint value'''
+        if not isinstance(lower_contraint_value, float) and  not isinstance(lower_contraint_value, int):
+            raise TypeError('lower_contraint_value must be float or int')
+        #end if
+        self._eqn_doc.Equations(self._eqn_index).LowerConstraint = lower_contraint_value
+        # 
+    @property
+    def upper_constraint(self) -> float:#-------------------------------------------------
+        ''' returns UpperContraint value'''
+        return self._eqn_doc.Equations(self._eqn_index).UpperConstraint
+        #
+    @upper_constraint.setter
+    def upper_constraint(self, upper_contraint_value:float):#-----------------------------
+        ''' sets UpperContraint value'''
+        if not isinstance(upper_contraint_value, float) and  not isinstance(upper_contraint_value, int):
+            raise TypeError('upper_contraint_value must be float or int')
+        #end if
+        self._eqn_doc.Equations(self._eqn_index).UpperConstraint = upper_contraint_value
+        #
+    @property
+    def step_size(self) -> float:#-------------------------------------------------
+        ''' returns StepSize value'''
+        return self._eqn_doc.Equations(self._eqn_index).StepSize
+        #
+    @step_size.setter
+    def step_size(self, step_size_value:float):#-----------------------------
+        ''' sets StepSize value'''
+        if not isinstance(step_size_value, float) and  not isinstance(step_size_value, int):
+            raise TypeError('step_size_value must be float or int')
+        #end if
+        self._eqn_doc.Equations(self._eqn_index).StepSize = step_size_value
+        #        
     def _update_equation(self):#---------------------------------------------------------
         '''write equation to AWR document'''
         variable_sign_dict = {'Variable definition':' = ', 'Parameter definition':' << ', 'Display value':':'}
