@@ -490,6 +490,9 @@ class _Measurement(_ProcessDefinitions):
         #
     @measurement_enabled.setter
     def measurement_enabled(self, enabled_state: bool):#----------------------------
+        if not isinstance(enabled_state, bool):
+            raise TypeError('enabled_state must be boolean type')
+        #end if        
         self._meas.Enabled = enabled_state
         #
     @property
